@@ -267,14 +267,16 @@ static int sa_inet_pton(int family, const char *strptr, void *addrptr)
            and for IPv6 inet_pton(3) would exist anyway */
         if (inet_aton(strptr, &in_val) == 0)
             return 0;
+        memcpy(addrptr, &in_val, sizeof(struct in_addr));
+        return 1;
 #elif defined(HAVE_INET_ADDR)
         /* at least for IPv4 try to rely on the even older inet_addr(3) */
         memset(&in_val, '\0', sizeof(in_val));
         if ((in_val.s_addr = inet_addr(strptr)) == ((in_addr_t)-1))
             return 0;
-#endif
         memcpy(addrptr, &in_val, sizeof(struct in_addr));
         return 1;
+#endif
     }
     errno = EAFNOSUPPORT;
     return 0;
@@ -732,7 +734,7 @@ sa_rc_t sa_addr_s2a(sa_addr_t *saa, const struct sockaddr *sabuf, socklen_t sale
 }
 
 /* export address object into URI */
-sa_rc_t sa_addr_a2u(sa_addr_t *saa, char **uri)
+sa_rc_t sa_addr_a2u(const sa_addr_t *saa, char **uri)
 {
     char uribuf[1024];
     struct sockaddr_un *un;
@@ -793,7 +795,7 @@ sa_rc_t sa_addr_a2u(sa_addr_t *saa, char **uri)
 }
 
 /* export address object into "struct sockaddr" */
-sa_rc_t sa_addr_a2s(sa_addr_t *saa, struct sockaddr **sabuf, socklen_t *salen)
+sa_rc_t sa_addr_a2s(const sa_addr_t *saa, struct sockaddr **sabuf, socklen_t *salen)
 {
     /* argument sanity check(s) */
     if (saa == NULL || sabuf == NULL || salen == 0)
@@ -808,7 +810,7 @@ sa_rc_t sa_addr_a2s(sa_addr_t *saa, struct sockaddr **sabuf, socklen_t *salen)
     return SA_OK;
 }
 
-sa_rc_t sa_addr_match(sa_addr_t *saa1, sa_addr_t *saa2, int prefixlen)
+sa_rc_t sa_addr_match(const sa_addr_t *saa1, const sa_addr_t *saa2, int prefixlen)
 {
     const unsigned char *ucp1, *ucp2;
     unsigned int uc1, uc2, mask;
@@ -934,7 +936,7 @@ sa_rc_t sa_addr_match(sa_addr_t *saa1, sa_addr_t *saa2, int prefixlen)
 }
 
 /* set timeouts if timeouts or done in kernel */
-static sa_rc_t sa_socket_settimeouts(sa_t *sa)
+static sa_rc_t sa_socket_settimeouts(const sa_t *sa)
 {
     /* stop processing if socket is still not allocated */
     if (sa->fdSocket == -1)
@@ -1222,7 +1224,7 @@ sa_rc_t sa_type(sa_t *sa, sa_type_t type)
 
     /* kill underlying socket if type changes */
     if (sa->eType != type)
-        sa_socket_kill(sa);
+        (void)sa_socket_kill(sa);
 
     /* set new type */
     sa->eType = type;
@@ -1421,7 +1423,7 @@ sa_rc_t sa_syscall(sa_t *sa, sa_syscall_t id, void (*fptr)(void), void *fctx)
 }
 
 /* bind socket to a local address */
-sa_rc_t sa_bind(sa_t *sa, sa_addr_t *laddr)
+sa_rc_t sa_bind(sa_t *sa, const sa_addr_t *laddr)
 {
     sa_rc_t rv;
     struct sockaddr_un *un;
@@ -1449,13 +1451,14 @@ sa_rc_t sa_bind(sa_t *sa, sa_addr_t *laddr)
 }
 
 /* connect socket to a remote address */
-sa_rc_t sa_connect(sa_t *sa, sa_addr_t *raddr)
+sa_rc_t sa_connect(sa_t *sa, const sa_addr_t *raddr)
 {
     int flags, n, error;
     fd_set rset, wset;
     socklen_t len;
     sa_rc_t rv;
     struct timeval *tv;
+    struct timeval tv_buf;
 
     /* argument sanity check(s) */
     if (sa == NULL || raddr == NULL)
@@ -1512,8 +1515,10 @@ sa_rc_t sa_connect(sa_t *sa, sa_addr_t *raddr)
     FD_ZERO(&wset);
     FD_SET(sa->fdSocket, &rset);
     FD_SET(sa->fdSocket, &wset);
-    if (!SA_TVISZERO(sa->tvTimeout[SA_TIMEOUT_CONNECT]))
-        tv = &sa->tvTimeout[SA_TIMEOUT_CONNECT];
+    if (!SA_TVISZERO(sa->tvTimeout[SA_TIMEOUT_CONNECT])) {
+        memcpy(&tv_buf, &sa->tvTimeout[SA_TIMEOUT_CONNECT], sizeof(struct timeval));
+        tv = &tv_buf;
+    }
     else
         tv = NULL;
     do {
@@ -1591,6 +1596,7 @@ sa_rc_t sa_accept(sa_t *sa, sa_addr_t **caddr, sa_t **csa)
 #endif
     } sa_buf;
     socklen_t sa_size;
+    struct timeval tv;
     int s;
     int i;
 
@@ -1610,9 +1616,9 @@ sa_rc_t sa_accept(sa_t *sa, sa_addr_t **caddr, sa_t **csa)
     if (!SA_TVISZERO(sa->tvTimeout[SA_TIMEOUT_ACCEPT])) {
         FD_ZERO(&fds);
         FD_SET(sa->fdSocket, &fds);
+        memcpy(&tv, &sa->tvTimeout[SA_TIMEOUT_ACCEPT], sizeof(struct timeval));
         do {
-            n = SA_SC_CALL_5(sa, select, sa->fdSocket+1, &fds, (fd_set *)NULL, (fd_set *)NULL,
-                             &sa->tvTimeout[SA_TIMEOUT_ACCEPT]);
+            n = SA_SC_CALL_5(sa, select, sa->fdSocket+1, &fds, (fd_set *)NULL, (fd_set *)NULL, &tv);
         } while (n == -1 && errno == EINTR);
         if (n == 0)
             return SA_RC(SA_ERR_TMT);
@@ -1763,6 +1769,7 @@ static int sa_read_raw(sa_t *sa, char *cpBuf, int nBufLen)
     int rv;
 #if !(defined(SO_RCVTIMEO) && defined(USE_SO_RCVTIMEO) && defined(SO_SNDTIMEO) && defined(USE_SO_SNDTIMEO))
     fd_set fds;
+    struct timeval tv;
 #endif
 
     /* if timeout is enabled, perform explicit/smart blocking instead
@@ -1771,9 +1778,9 @@ static int sa_read_raw(sa_t *sa, char *cpBuf, int nBufLen)
     if (!SA_TVISZERO(sa->tvTimeout[SA_TIMEOUT_READ])) {
         FD_ZERO(&fds);
         FD_SET(sa->fdSocket, &fds);
+        memcpy(&tv, &sa->tvTimeout[SA_TIMEOUT_READ], sizeof(struct timeval));
         do {
-            rv = SA_SC_CALL_5(sa, select, sa->fdSocket+1, &fds, (fd_set *)NULL, (fd_set *)NULL,
-                              &sa->tvTimeout[SA_TIMEOUT_READ]);
+            rv = SA_SC_CALL_5(sa, select, sa->fdSocket+1, &fds, (fd_set *)NULL, (fd_set *)NULL, &tv);
         } while (rv == -1 && errno == EINTR);
         if (rv == 0) {
             errno = ETIMEDOUT;
@@ -1942,6 +1949,7 @@ static int sa_write_raw(sa_t *sa, const char *cpBuf, int nBufLen)
     int rv;
 #if !(defined(SO_RCVTIMEO) && defined(USE_SO_RCVTIMEO) && defined(SO_SNDTIMEO) && defined(USE_SO_SNDTIMEO))
     fd_set fds;
+    struct timeval tv;
 #endif
 
     /* if timeout is enabled, perform explicit/smart blocking instead
@@ -1950,9 +1958,9 @@ static int sa_write_raw(sa_t *sa, const char *cpBuf, int nBufLen)
     if (!SA_TVISZERO(sa->tvTimeout[SA_TIMEOUT_WRITE])) {
         FD_ZERO(&fds);
         FD_SET(sa->fdSocket, &fds);
+        memcpy(&tv, &sa->tvTimeout[SA_TIMEOUT_WRITE], sizeof(struct timeval));
         do {
-            rv = SA_SC_CALL_5(sa, select, sa->fdSocket+1, (fd_set *)NULL, &fds, (fd_set *)NULL,
-                              &sa->tvTimeout[SA_TIMEOUT_WRITE]);
+            rv = SA_SC_CALL_5(sa, select, sa->fdSocket+1, (fd_set *)NULL, &fds, (fd_set *)NULL, &tv);
         } while (rv == -1 && errno == EINTR);
         if (rv == 0) {
             errno = ETIMEDOUT;
@@ -2125,7 +2133,7 @@ sa_rc_t sa_flush(sa_t *sa)
 }
 
 /* shutdown a socket connection */
-sa_rc_t sa_shutdown(sa_t *sa, char *flags)
+sa_rc_t sa_shutdown(sa_t *sa, const char *flags)
 {
     int how;
 
@@ -2177,6 +2185,7 @@ sa_rc_t sa_recv(sa_t *sa, sa_addr_t **raddr, char *buf, size_t buflen, size_t *b
     ssize_t n;
     int k;
     fd_set fds;
+    struct timeval tv;
 
     /* argument sanity check(s) */
     if (sa == NULL || buf == NULL || buflen == 0 || raddr == NULL)
@@ -2195,9 +2204,9 @@ sa_rc_t sa_recv(sa_t *sa, sa_addr_t **raddr, char *buf, size_t buflen, size_t *b
     if (!SA_TVISZERO(sa->tvTimeout[SA_TIMEOUT_READ])) {
         FD_ZERO(&fds);
         FD_SET(sa->fdSocket, &fds);
+        memcpy(&tv, &sa->tvTimeout[SA_TIMEOUT_READ], sizeof(struct timeval));
         do {
-            k = SA_SC_CALL_5(sa, select, sa->fdSocket+1, &fds, (fd_set *)NULL, (fd_set *)NULL,
-                             &sa->tvTimeout[SA_TIMEOUT_READ]);
+            k = SA_SC_CALL_5(sa, select, sa->fdSocket+1, &fds, (fd_set *)NULL, (fd_set *)NULL, &tv);
         } while (k == -1 && errno == EINTR);
         if (k == 0)
             errno = ETIMEDOUT;
@@ -2233,6 +2242,7 @@ sa_rc_t sa_send(sa_t *sa, sa_addr_t *raddr, const char *buf, size_t buflen, size
     int k;
     fd_set fds;
     sa_rc_t rv;
+    struct timeval tv;
 
     /* argument sanity check(s) */
     if (sa == NULL || buf == NULL || buflen == 0 || raddr == NULL)
@@ -2252,9 +2262,9 @@ sa_rc_t sa_send(sa_t *sa, sa_addr_t *raddr, const char *buf, size_t buflen, size
     if (!SA_TVISZERO(sa->tvTimeout[SA_TIMEOUT_WRITE])) {
         FD_ZERO(&fds);
         FD_SET(sa->fdSocket, &fds);
+        memcpy(&tv, &sa->tvTimeout[SA_TIMEOUT_WRITE], sizeof(struct timeval));
         do {
-            k = SA_SC_CALL_5(sa, select, sa->fdSocket+1, (fd_set *)NULL, &fds, (fd_set *)NULL,
-                             &sa->tvTimeout[SA_TIMEOUT_WRITE]);
+            k = SA_SC_CALL_5(sa, select, sa->fdSocket+1, (fd_set *)NULL, &fds, (fd_set *)NULL, &tv);
         } while (k == -1 && errno == EINTR);
         if (k == 0)
             errno = ETIMEDOUT;
