@@ -168,11 +168,11 @@ extern sa_rc_t sa_addr_create  (sa_addr_t **__saa);
 extern sa_rc_t sa_addr_destroy (sa_addr_t  *__saa);
 
 /* address operations */
-extern sa_rc_t sa_addr_u2a     (sa_addr_t  *__saa, const char *__uri, ...);
-extern sa_rc_t sa_addr_s2a     (sa_addr_t  *__saa, const struct sockaddr *__sabuf, socklen_t __salen);
-extern sa_rc_t sa_addr_a2u     (sa_addr_t  *__saa, char **__uri);
-extern sa_rc_t sa_addr_a2s     (sa_addr_t  *__saa, struct sockaddr **__sabuf, socklen_t *__salen);
-extern sa_rc_t sa_addr_match   (sa_addr_t  *__saa1, sa_addr_t *__saa2, int __prefixlen);
+extern sa_rc_t sa_addr_u2a     (sa_addr_t       *__saa, const char *__uri, ...);
+extern sa_rc_t sa_addr_s2a     (sa_addr_t       *__saa, const struct sockaddr *__sabuf, socklen_t __salen);
+extern sa_rc_t sa_addr_a2u     (const sa_addr_t *__saa, char **__uri);
+extern sa_rc_t sa_addr_a2s     (const sa_addr_t *__saa, struct sockaddr **__sabuf, socklen_t *__salen);
+extern sa_rc_t sa_addr_match   (const sa_addr_t *__saa1, const sa_addr_t *__saa2, int __prefixlen);
 
 /* socket object operations */
 extern sa_rc_t sa_create       (sa_t **__sa);
@@ -186,14 +186,14 @@ extern sa_rc_t sa_option       (sa_t  *__sa, sa_option_t  __id, ...);
 extern sa_rc_t sa_syscall      (sa_t  *__sa, sa_syscall_t __id, void (*__fptr)(void), void *__fctx);
 
 /* socket connection operations */
-extern sa_rc_t sa_bind         (sa_t  *__sa, sa_addr_t  *__laddr);
-extern sa_rc_t sa_connect      (sa_t  *__sa, sa_addr_t  *__raddr);
+extern sa_rc_t sa_bind         (sa_t  *__sa, const sa_addr_t *__laddr);
+extern sa_rc_t sa_connect      (sa_t  *__sa, const sa_addr_t *__raddr);
 extern sa_rc_t sa_listen       (sa_t  *__sa, int __backlog);
 extern sa_rc_t sa_accept       (sa_t  *__sa, sa_addr_t **__caddr, sa_t **__csa);
 extern sa_rc_t sa_getremote    (sa_t  *__sa, sa_addr_t **__raddr);
 extern sa_rc_t sa_getlocal     (sa_t  *__sa, sa_addr_t **__laddr);
 extern sa_rc_t sa_getfd        (sa_t  *__sa, int *__fd);
-extern sa_rc_t sa_shutdown     (sa_t  *__sa, char *__flags);
+extern sa_rc_t sa_shutdown     (sa_t  *__sa, const char *__flags);
 
 /* socket input/output operations (stream communication) */
 extern sa_rc_t sa_read         (sa_t  *__sa, char       *__buf, size_t __buflen, size_t *__bufdone);
